package Package1;


import java.io.BufferedReader;
import java.io.InputStreamReader;

public class DiatonicTranspose {

    static int decodePc(int currentPitch) {
        return ((currentPitch % 1000) / 10);
    }

    static int decodeNc(int currentPitch) {
        return (currentPitch % 10);
    }

    static int decodeOctave(int currentPitch) {
        return currentPitch / 1000;
    }

    static int[] diatonicTranspose(int[] sequence, int amount) {
        int[] halftoneOffset = {0, 2, 4, 5, 7, 9, 11};
        int[] response = new int[sequence.length];
        for (int i = 0; i < sequence.length; i++) {
            int currentCbr = sequence[i];
            // calcolo dell'ottava
            int newOctave = decodeOctave(currentCbr) + amount / 7;
            // calcolo della trasposizione a meno delle ottave
            int currentAmount = amount - 7 * (amount / 7);
            // calcolo dell'nc
            int currentNc = decodeNc(currentCbr);
            // calcolo del nuovo NC a meno delle ottave
            int newNc = currentNc + currentAmount;
            // oltre i limiti 0..7: ricalcolo NC e modifica nuova ottava
            if (newNc > 6) {
                newNc -= 7;
                newOctave++;
            } else if (newNc < 0) {
                newNc += 7;
                newOctave--;
            }
            // calcolo del pc
            int currentPc = decodePc(currentCbr);
            // spiazzamento rispetto alla nota naturale originale + spiazzamento nuova nota naturale
            int newPc = currentPc - halftoneOffset[currentNc] + halftoneOffset[newNc];
            // oltre i limiti 0..11: ricalcolo PC
            if (newPc > 11)
                newPc -= 12;
            else if (newPc < 0)
                newPc += 12;
            // ricostruzione del valore cbr
            response[i] = newOctave * 1000 + newPc * 10 + newNc;
        }
        return response;
    }

    public static void main(String[] args) {

        if (args.length < 1) {
            System.out.println("E' necessario immettere una serie di valori espressi in codifica CBR");
            System.exit(1);
        }

        int[] cbrSequence = new int[args.length];
        try {
            for (int i = 0; i < args.length; i++) {
                cbrSequence[i] = Integer.parseInt(args[i]);
                // verifica sulla correttezza della codifica cbr in ingresso:
                //   nc                            pc
                if ((cbrSequence[i] % 10 > 6) || ((cbrSequence[i] % 1000) / 10) > 11) {
                    System.out.println("Valore immesso non valido nella codifica CBR: " + cbrSequence[i]);
                    System.exit(1);
                }
            }
        } catch (NumberFormatException e) {
            System.out.println("Gli argomenti devono essere di tipo intero");
            System.exit(1);
        }

        // Standard input
        BufferedReader br = new BufferedReader(new InputStreamReader(System.in));

        // Trasposizione diatonica
        System.out.println("TRASPOSIZIONE DIATONICA di ");
        for (int candidate : cbrSequence)
            System.out.print(candidate + " ");
        System.out.println();

        // Lettura numero di gradi
        int diatonicAmount = 0;
        try {
            System.out.print("Inserisci il numero di gradi della scala: ");
            diatonicAmount = Integer.valueOf(br.readLine());
        } catch (Exception e) {
            System.out.println("Errore");
            System.exit(1);
        }
        int[] results = diatonicTranspose(cbrSequence, diatonicAmount);

        // Output risultati
        for (int candidate : results)
            System.out.print(candidate + " ");
        System.out.println();

        System.out.println();
        System.exit(0);
    }
}
