package Package1;

import java.io.BufferedWriter;
import java.io.File;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Random;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.OutputKeys;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

enum pitchEnum {
    A, B, C, D, E, F, G, rest, unknown
}

enum accidentalEnum {
    none, flat, natural, sharp
}

class Note {

    public pitchEnum pitch;
    public accidentalEnum accidental;
    public int octave;
    public int den;

    private Note() {
        pitch = pitchEnum.unknown;
        accidental = accidentalEnum.none;
        octave = 0;
        den = 1;
    }

    public static Note RandomNote() {
        Note n = new Note();
        Random randomGenerator = new Random();

        // randomGenerator.nextInt(n) ottiene un numero intero compreso tra 0 e (n-1)
        int pitchIndex = randomGenerator.nextInt(pitchEnum.values().length - 1);
        n.pitch = pitchEnum.values()[pitchIndex];
        n.den = (int) Math.pow(2, randomGenerator.nextInt(7));
        if (pitchIndex < pitchEnum.values().length - 1) {
            n.accidental = accidentalEnum.values()[randomGenerator.nextInt(accidentalEnum.values().length)];
            n.octave = randomGenerator.nextInt(7);
        }
        return n;
    }
}

public class SaveMelody {

    public static void main(String[] args) {

        // compilazione della struttura dati con 30 istanze casuali di Note
        ArrayList<Note> melody = new ArrayList<>();
        for (int i = 0; i < 30; i++)
            melody.add(Note.RandomNote());

        // creazione nuovo documento
        DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
        DocumentBuilder docBuilder = null;
        try {
            docBuilder = docFactory.newDocumentBuilder();
        } catch (ParserConfigurationException ex) {
        }
        Document doc = docBuilder.newDocument();

        // elemento radice
        Element root = doc.createElement("melodia");
        doc.appendChild(root);

        for (Note n : melody) {
            Element note = null;
            if (n.pitch != pitchEnum.rest) {
                // creazione elemento nota 
                // NOTA: occorre sempre prima creare e poi aggiungere
                note = doc.createElement("nota");
                
                Element pitch = doc.createElement("altezza");
                pitch.setAttribute("nome", n.pitch.toString());
                pitch.setAttribute("ottava", Integer.toString(n.octave));
                if (n.accidental != accidentalEnum.none)
                    pitch.setAttribute("alterazione", n.accidental.toString());
                note.appendChild(pitch);
            } else
                note = doc.createElement("pausa");
            Element duration = doc.createElement("durata");
            duration.setAttribute("num", "1");
            duration.setAttribute("den", Integer.toString(n.den));
            note.appendChild(duration);
            // aggiunta dell'elemento nota (con tutti i sottoelementi/attributi) alla radice
            root.appendChild(note);
        }

        // salvataggio
        try {
            Result output = new StreamResult(new File("melodia.xml"));
            Source input = new DOMSource(doc);

            TransformerFactory tf = TransformerFactory.newInstance();
            Transformer t = tf.newTransformer();
            t.setOutputProperty(OutputKeys.INDENT, "yes");
            t.setOutputProperty("{http://xml.apache.org/xslt}indent-amount", "3");

            t.transform(input, output);
        } catch (TransformerException ex) {
            System.out.println("Salvataggio non riuscito");
            System.exit(1);
        }

        // Alternativa basata su uso stringhe (apparentemente piu' semplice ma rischiosa e applicabile solo a casi banali)
        // Ha scopo solo dimostrativo!!!
        String result = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>";
        result += "<melodia>";
        for (Note n : melody) {
            if (n.pitch != pitchEnum.rest) {
                result += "<nota>";
                result += "<altezza nome='" + n.pitch.toString() + "' ottava='" + n.octave + "'";
                if (n.accidental != accidentalEnum.none)
                    result += " alterazione='" + n.accidental + "'";
                result += "/>";
            } else
                result += "<pausa>";
            result += "<durata num='1' den='" + n.den + "' />";
            if (n.pitch != pitchEnum.rest)
                result += "</nota>";
            else
                result += "</pausa>";
        }
        result += "</melodia>";
        BufferedWriter writer = null;
        try {
            writer = new BufferedWriter(new FileWriter("melodia2.xml"));
            writer.write(result);
        } catch (IOException e) {
        } finally {
            try {
                if (writer != null)
                    writer.close();
            } catch (IOException e) {
            }
        }
    }
}
