package Package4;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;

enum PitchEnum {
    A, B, C, D, E, F, G, Rest, unknown
}

enum AccidentalEnum {
    FLAT, NATURAL, SHARP
}

class Note {

    private PitchEnum pitch = PitchEnum.unknown;
    private AccidentalEnum accidental = AccidentalEnum.NATURAL;
    private int num = 0;
    private final int den = 8;
    private int octave = -1;

    public void setOctave(String octave) {
        try {
            this.octave = Integer.valueOf(octave);
        } catch (NumberFormatException e) {
            this.octave = 0;
        }
    }

    public void setPitch(char pitch) {
        switch (pitch) {
            case 'A':
                this.pitch = PitchEnum.A;
                break;
            case 'B':
                this.pitch = PitchEnum.B;
                break;
            case 'C':
                this.pitch = PitchEnum.C;
                break;
            case 'D':
                this.pitch = PitchEnum.D;
                break;
            case 'E':
                this.pitch = PitchEnum.E;
                break;
            case 'F':
                this.pitch = PitchEnum.F;
                break;
            case 'G':
                this.pitch = PitchEnum.G;
                break;
            case 'R':
                this.pitch = PitchEnum.Rest;
                break;
            default:
                this.pitch = PitchEnum.unknown;
        }
    }

    public void setAccidental(char accidental) {
        switch (accidental) {
            case '#':
                this.accidental = AccidentalEnum.SHARP;
                break;
            case 'b':
                this.accidental = AccidentalEnum.FLAT;
                break;
            default:
                this.accidental = AccidentalEnum.NATURAL;
        }
    }

    public void AddTimeGranule() {
        num++;
    }
    
    public int getLengthInTimeGranules() {
        return num;
    }

    @Override
    public String toString() {
        if (pitch == PitchEnum.unknown)
            return "";
        String result = pitch.toString();
        switch (accidental) {
            case SHARP:
                result += "#";
                break;
            case FLAT:
                result += "b";
                break;
        }
        if (pitch != PitchEnum.Rest)
            result += octave;
        result += " [" + num + "/" + den + "]";
        return result;
    }
}

public class EasyScore {

    public static void main(String[] args) {
        ArrayList<ArrayList<Note>> score = new ArrayList<>();

        if (args.length != 1) {
            System.out.println("Numero di argomenti errato. Inserisci il nome del file!");
            System.exit(1);
        }
        File name = new File(args[0]);
        if (!name.isFile()) {
            System.out.println("Il file immesso non esiste o non e' valido. Inserisci il nome del file!");
            System.exit(1);
        }

        try (BufferedReader input = new BufferedReader(new FileReader(name))) {
            String line;
            // lettura del file di testo riga per riga
            while ((line = input.readLine()) != null) {
                ArrayList<Note> notes = new ArrayList<>();
                String[] splitted = line.split(",");
                Note currentNote = new Note();
                for (String s : splitted) {
                    String currentSymbol = s.trim(); // Rimuove eventuali spazi iniziali/finali
                    if (currentSymbol.length() == 0)
                        currentNote.AddTimeGranule();
                    else {
                        currentNote = new Note();
                        currentNote.setPitch(currentSymbol.charAt(0));
                        currentSymbol = currentSymbol.substring(1);
                        if (currentSymbol.startsWith("#") || currentSymbol.startsWith("b")) {
                            currentNote.setAccidental(currentSymbol.charAt(0));
                            currentSymbol = currentSymbol.substring(1);
                        }
                        currentNote.setOctave(currentSymbol);
                        currentNote.AddTimeGranule();
                        notes.add(currentNote);
                    }
                }
                score.add(notes);
            }
        } catch (IOException e) {
        }

        for (int i = 0; i < score.size(); i++) {
            System.out.println("\nStrumento " + (i + 1));
            for (int j = 0; j < score.get(i).size(); j++)
                System.out.println(score.get(i).get(j).toString());
        }
        
        int colWidth = 12;
        try (FileWriter w = new FileWriter("easyscore.txt");
          BufferedWriter bw = new BufferedWriter(w)) {
            for (int i = 0; i < score.size(); i++) {
                for (int j = 0; j < score.get(i).size(); j++) {
                    String outString = score.get(i).get(j).toString();
                    bw.write(outString);
                    for (int k = outString.length(); k < colWidth; k++)
                        bw.write(' ');
                    for (int k = 1; k < score.get(i).get(j).getLengthInTimeGranules(); k++)
                        for (int l = 0; l < colWidth; l++)
                            bw.write(' ');
                }
                bw.newLine();
            }
        } catch (IOException ex) {
        }
    }
}
